// (C) 2019-2025 GoodData Corporation
import { isNoDataError, isUnexpectedResponseError, } from "@gooddata/sdk-backend-spi";
import { defFingerprint } from "@gooddata/sdk-model";
import React from "react";
import { injectIntl } from "react-intl";
import noop from "lodash/noop.js";
import omit from "lodash/omit.js";
import isEqual from "lodash/isEqual.js";
import { DataTooLargeToDisplaySdkError, NegativeValuesSdkError, ForecastNotReceivedSdkError, isForecastNotReceived, ClusteringNotReceivedSdkError, isClusteringNotReceived, } from "../../errors/GoodDataSdkError.js";
import { createExportErrorFunction, createExportFunction } from "../../vis/export.js";
import { DataViewFacade } from "../../results/facade.js";
import { convertError } from "../../errors/errorHandling.js";
import { IntlWrapper } from "../../localization/IntlWrapper.js";
import { getAvailableDrillTargets } from "./availableDrillTargets.js";
/**
 * A HOC to wrap data visualization components with loading / error handling.
 *
 * Note: this is a legacy HOC with a long history. In v7 we had VisualizationLoadingHOC - that one was used for
 * all components and was linked to AFM and the paging and everything. We took this and gutted it out, changed to
 * work with executions and to only support reading all the data.
 *
 * @param InnerComponent - component to wrap
 * @internal
 */
export function withEntireDataView(InnerComponent) {
    class LoadingHOCWrapped extends React.Component {
        static defaultProps = InnerComponent.defaultProps || {};
        hasUnmounted = false;
        abortController;
        /**
         * Fingerprint of the last execution definition the initialize was called with.
         */
        lastInitRequestFingerprint = null;
        constructor(props) {
            super(props);
            this.state = {
                isLoading: false,
                error: null,
                executionResult: null,
                dataView: null,
            };
            this.onLoadingChanged = this.onLoadingChanged.bind(this);
            this.onDataTooLarge = this.onDataTooLarge.bind(this);
            this.onNegativeValues = this.onNegativeValues.bind(this);
            this.onDataView = this.onDataView.bind(this);
            this.abortController = new AbortController();
        }
        componentDidMount() {
            this.initDataLoading(this.props.execution, this.props.forecastConfig, this.props.clusteringConfig);
        }
        render() {
            const { isLoading, error, dataView } = this.state;
            const { intl } = this.props;
            // lower-level components do not need workspace
            const props = this.stripWorkspace(this.props);
            return (React.createElement(InnerComponent, { ...props, dataView: dataView, onDataTooLarge: this.onDataTooLarge, onNegativeValues: this.onNegativeValues, error: error, isLoading: isLoading, intl: intl }));
        }
        UNSAFE_componentWillReceiveProps(nextProps) {
            //  we need strict equality here in case only the buckets changed (not measures or attributes)
            if (!this.props.execution.equals(nextProps.execution) ||
                !isEqual(this.props.forecastConfig, nextProps.forecastConfig) ||
                !isEqual(this.props.clusteringConfig, nextProps.clusteringConfig)) {
                this.refreshAbortController();
                this.initDataLoading(nextProps.execution, nextProps.forecastConfig, nextProps.clusteringConfig);
            }
        }
        componentWillUnmount() {
            this.hasUnmounted = true;
            this.onLoadingChanged = noop;
            this.onDataView = noop;
            this.onError = noop;
            this.refreshAbortController();
        }
        refreshAbortController() {
            if (this.props.enableExecutionCancelling) {
                if (this.state.isLoading) {
                    this.abortController.abort();
                }
                this.abortController = new AbortController();
            }
        }
        onLoadingChanged(loadingState) {
            const { onLoadingChanged } = this.props;
            onLoadingChanged?.(loadingState);
            const { isLoading } = loadingState;
            const state = { isLoading };
            if (isLoading) {
                state.error = null;
            }
            this.setState(state);
        }
        onDataView(dataView) {
            const { onDataView } = this.props;
            onDataView?.(DataViewFacade.for(dataView));
        }
        onError(error) {
            const { onExportReady } = this.props;
            if (!isForecastNotReceived(error) && !isClusteringNotReceived(error)) {
                const err = error;
                this.setState({ error: err.getMessage(), dataView: null });
            }
            this.onLoadingChanged({ isLoading: false });
            if (onExportReady) {
                onExportReady(createExportErrorFunction(error));
            }
            this.props.onError?.(error);
        }
        onDataTooLarge(_data, errorMessage) {
            this.onError(new DataTooLargeToDisplaySdkError(errorMessage));
        }
        onNegativeValues() {
            this.onError(new NegativeValuesSdkError());
        }
        async initDataLoading(originalExecution, forecastConfig, clusteringConfig) {
            let execution = originalExecution;
            if (this.props.enableExecutionCancelling) {
                execution = execution.withSignal(this.abortController.signal);
            }
            const { onExportReady, pushData, exportTitle } = this.props;
            this.onLoadingChanged({ isLoading: true });
            this.setState({ dataView: null });
            this.lastInitRequestFingerprint = defFingerprint(execution.definition);
            try {
                const executionResult = await execution.execute();
                if (this.lastInitRequestFingerprint !== defFingerprint(execution.definition)) {
                    return;
                }
                if (this.hasUnmounted) {
                    return;
                }
                const originalDataView = await executionResult.readAll().catch((err) => {
                    /**
                     * When execution result is received successfully,
                     * but data load fails with unexpected http response,
                     * we still want to push availableDrillTargets
                     */
                    if (isUnexpectedResponseError(err) && pushData) {
                        const availableDrillTargets = getAvailableDrillTargets(DataViewFacade.forResult(executionResult));
                        pushData({ availableDrillTargets });
                    }
                    throw err;
                });
                if (this.hasUnmounted) {
                    return;
                }
                if (this.lastInitRequestFingerprint !== defFingerprint(originalDataView.definition)) {
                    /*
                     * Stop right now if the data are not relevant anymore because there was another
                     * initialize request in the meantime.
                     */
                    return;
                }
                let dataView = originalDataView;
                if (forecastConfig) {
                    dataView = originalDataView.withForecast(forecastConfig);
                }
                if (clusteringConfig) {
                    dataView = originalDataView.withClustering(clusteringConfig);
                    try {
                        const clusteringResult = await executionResult.readClusteringAll(clusteringConfig);
                        dataView = dataView.withClustering(clusteringConfig, clusteringResult);
                    }
                    catch (e) {
                        dataView = dataView.withClustering(clusteringConfig, {
                            attribute: [],
                            clusters: [],
                            xcoord: [],
                            ycoord: [],
                        });
                        const err = e;
                        throw new ClusteringNotReceivedSdkError(err.responseBody?.reason || err.message || "Unknown error", err);
                    }
                }
                this.setState({ dataView, error: null, executionResult });
                this.onLoadingChanged({ isLoading: false });
                this.onDataView(dataView);
                if (onExportReady) {
                    onExportReady(createExportFunction(dataView.result, exportTitle));
                }
                if (pushData) {
                    const availableDrillTargets = getAvailableDrillTargets(DataViewFacade.for(dataView));
                    pushData({ dataView, availableDrillTargets });
                }
                if (this.hasUnmounted) {
                    return;
                }
                if (dataView.forecastConfig && forecastConfig) {
                    try {
                        const forecastResult = await executionResult.readForecastAll(dataView.forecastConfig);
                        const updatedDataView = dataView.withForecast(dataView.forecastConfig, forecastResult);
                        this.setState((s) => ({ ...s, dataView: updatedDataView }));
                        if (pushData) {
                            pushData({
                                dataView: updatedDataView,
                                propertiesMeta: {
                                    slicedForecast: forecastConfig.forecastPeriod !==
                                        dataView.forecastConfig?.forecastPeriod,
                                },
                            });
                        }
                    }
                    catch (e) {
                        const updatedDataView = dataView.withForecast(undefined);
                        this.setState((s) => ({ ...s, dataView: updatedDataView }));
                        if (pushData) {
                            pushData({ dataView: updatedDataView });
                        }
                        const err = e;
                        throw new ForecastNotReceivedSdkError(err.responseBody?.reason || err.message || "Unknown error", err);
                    }
                }
            }
            catch (error) {
                if (this.lastInitRequestFingerprint !== defFingerprint(execution.definition)) {
                    return;
                }
                if (this.hasUnmounted) {
                    return;
                }
                /*
                 * There can be situations, where there is no data to visualize but the result / dataView contains
                 * metadata essential for setup of drilling. Look for that and if available push up.
                 */
                if (isNoDataError(error) && error.dataView && pushData) {
                    const availableDrillTargets = getAvailableDrillTargets(DataViewFacade.for(error.dataView));
                    pushData({ availableDrillTargets });
                }
                this.onError(convertError(error));
            }
        }
        stripWorkspace = (props) => {
            return omit(props, ["workspace"]);
        };
    }
    const IntlLoadingHOC = injectIntl(LoadingHOCWrapped);
    return class LoadingHOC extends React.Component {
        render() {
            return (React.createElement(IntlWrapper, { locale: this.props.locale },
                React.createElement(IntlLoadingHOC, { ...this.props })));
        }
    };
}
//# sourceMappingURL=withEntireDataView.js.map